﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.IO;
using System.Runtime.Serialization;

namespace EntityDocumentation
{
    class Helper
    {
        private static string _productBinPath;
        /// <summary>
        /// Return the product's bin folder path
        /// </summary>
        public static string ProductBinPath 
        {
            get
            {
                if (!string.IsNullOrEmpty(_productBinPath))
                {
                    int i = _productBinPath.LastIndexOf("\\");
                    if (i > 0 && i+1 == _productBinPath.Length) _productBinPath= _productBinPath.Substring(0, i);
                }
                return _productBinPath; 
            }
            set
            { 
                _productBinPath = value; 
            }
        }

        /// <summary>
        /// Check if Product path is a valid path
        /// </summary>
        public static bool IsProductPathValid
        {
            get 
            {
                return System.IO.Directory.Exists(ProductBinPath);
            }
        }

        /// <summary>
        /// Return the product's xmd folder path
        /// </summary>
        public static string ProductXmdPath 
        { 
            get 
            {
                return string.Format("{0}\\xmd", ProductDomainPath);
            } 
        }

        /// <summary>
        /// Return the product's domain path
        /// </summary>
        public static string ProductDomainPath 
        { 
            get 
            {
                return ProductBinPath.Substring(0, ProductBinPath.LastIndexOf("\\bin"));
            } 
        }

        /// <summary>
        /// Return documentation domain path
        /// </summary>
        public static string DocumentationDomainPath
        {
            get
            {
                return AppDomain.CurrentDomain.BaseDirectory.Substring(0, AppDomain.CurrentDomain.BaseDirectory.LastIndexOf("\\bin\\"));
            }
        }
        
        /// <summary>
        /// Return Documentation html path
        /// </summary>
        public static string DocumentationHtmlPath 
        { 
            get 
            {
                return string.Format("{0}\\html\\", DocumentationDomainPath);
            } 
        }

        /// <summary>
        /// Return Documentation xmd path
        /// </summary>
        public static string DocumentationXmdPath
        {
            get
            {
                return string.Format("{0}\\xmd\\", DocumentationDomainPath);
            }
        }

        /// <summary>
        /// Return product prefix.
        /// </summary>
        public static string ProductPrefix
        {
            get
            {
                if (File.Exists(string.Format("{0}\\Exact.Entity.ER.ESE.dll", Helper.ProductBinPath)))
                    return "ESE";
                else if (File.Exists(string.Format("{0}\\Exact.Entity.Service.EG.dll", Helper.ProductBinPath)))
                    return "EG";
                else
                    return string.Empty;
            }
        }

        #region Product releases
        public static void SetProductRelease()
        {
            string filename = string.Empty;
            switch (ProductPrefix)
            {
                case "ESE":
                    filename = string.Format("{0}\\version.config", ProductDomainPath);
                    if (File.Exists(filename))
                    {
                        System.Xml.XmlDocument x = new System.Xml.XmlDocument();
                        x.Load(filename);
                        foreach (var node in x.ChildNodes)
                        {
                            if (node is System.Xml.XmlElement)
                            {
                                System.Xml.XmlElement element = (System.Xml.XmlElement)node;
                                foreach (var node2 in element.ChildNodes)
                                {
                                    System.Xml.XmlElement element2 = (System.Xml.XmlElement)node2;
                                    switch (element2.Name)
                                    {
                                        case "release":
                                            productRelease = element2.InnerText;
                                            return;
                                        default: break;
                                    }
                                }

                            }
                        }
                    }
                    break;
                case "EG":
                    filename = string.Format("{0}\\baco.ini", ProductBinPath);
                    if (File.Exists(filename))
                    {
                        productRelease = File.ReadLines(filename).FirstOrDefault(line => line.StartsWith("CurrentBatch"));
                        productRelease = productRelease.Split('=')[1];
                    }
                    break;
                default: break;
            }
        }

        private static string productRelease = string.Empty;
        public static string ProductRelease
        {
            get
            {
                if (string.IsNullOrEmpty(productRelease)) SetProductRelease();
                return productRelease;
            }
        }
        #endregion

        public static Data.DocumentHeader Header = new Data.DocumentHeader();

        /// <summary>
        /// Read configuration
        /// </summary>
        /// <typeparam name="T">Configuration Template</typeparam>
        /// <param name="config">Configuration type</param>
        /// <param name="fileName">file to be read</param>
        public static bool ReadSetting<T>(ref T config, string fileName) where T : new()
        {
            Type type = typeof(T);
            if (string.IsNullOrEmpty(fileName))
            {
                config = new T();
                return true;
            }

            FileStream reader = null;
            try
            {
                DataContractSerializer serializer = new DataContractSerializer(type);
                reader = new FileStream(fileName, FileMode.OpenOrCreate);
                config = (T)serializer.ReadObject(reader);
            }
            catch (UnauthorizedAccessException ex1)
            {
                config = new T();

                string error = string.Format("{0}{1}{1}The following could be the possible cause of the error{1}1) File could be in read only mode ({2}){1}2) File is not checked out from TFS ({3}){1}Application will be closed to avoid data damage.", 
                    ex1.Message,Environment.NewLine,fileName, fileName.Substring(fileName.LastIndexOf("\\")+1));

                System.Windows.MessageBox.Show(error, "Error", System.Windows.MessageBoxButton.OK, System.Windows.MessageBoxImage.Error);
                return false;
            }
            catch (Exception)
            {
                config = new T();
            }
            finally { if (reader != null) reader.Close(); }
            return true;
        }

        /// <summary>
        /// Write configuration
        /// </summary>
        /// <typeparam name="T">Configuration template</typeparam>
        /// <param name="config">Configuration type</param>
        /// <param name="fileName">file to be read</param>
        public static void WriteSetting<T>(T config, string fileName) where T : new()
        {
            Type type = typeof(T);
            if (string.IsNullOrEmpty(fileName)) return;

            FileStream writer = null;
            try
            {
                DataContractSerializer serializer = new DataContractSerializer(type);
                writer = new FileStream(fileName, FileMode.OpenOrCreate | FileMode.Truncate);
                serializer.WriteObject(writer, config);
            }
            catch (Exception ex) { throw ex; }
            finally { if (writer != null) writer.Close(); }

        }
    }
}
