﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Shapes;
using System.IO;

namespace EntityDocumentation.Overview
{
    /// <summary>
    /// Interaction logic for ImportFromServiceLocator.xaml
    /// </summary>
    public partial class ImportFromServiceLocator : Window
    {

        private Data.ProgressBar solutionProgress = null;
        private Data.ProgressBar entityProgress = null;

        #region Constructor
        public ImportFromServiceLocator()
        {
            InitializeComponent();
        }

        private void Window_Loaded(object sender, RoutedEventArgs e)
        {
            List<string> serviceLocator = new List<string>();
            foreach (string s in Directory.GetFiles(Helper.ProductXmdPath, "exact.servicelocation*.config"))    // Get available service location
                serviceLocator.Add(s.Replace(String.Format("{0}\\", Helper.ProductDomainPath), ""));            // Show all available service location.
            lst.ItemsSource = serviceLocator;

            // Initialize progress bars
            solutionProgress = new Data.ProgressBar(progressBarSolution, progressTextSolution); 
            entityProgress = new Data.ProgressBar(progressBarEntity, progressTextEntity);
        }
        #endregion

        #region entity
        // Add new entity to documentation. Prefill with defaults based on available information.
        private void AddNewEntity(Data.Entity data, Exact.Services.Utilities.Locator.Configuration.Service service, Data.SolutionArea solutionArea, Exact.Services.MetaModel.Entity.Metadata meta, StringBuilder sb, string s)
        {
            // new set of entity and properties for specific solution area.
            data = new Data.Entity { Name = service.Name, EntitiesProviderName = service.EntitiesOperationsProviderName, FixedCriteria = service.FixedCriteriaPath, EntityProviderName = service.OperationsProviderName, MetadataProviderName = service.MetadataProviderName };
            data.PropertyList = new Data.PropertyList();
            solutionArea.EntityList.Add(data);
            try
            {   // Prefill with defaults based on configuration available information.

                data.IsMetadataGenerated = (service.OperationsProviderName == "Exact.EntityOperations.ESE" || service.OperationsProviderName == "Exact.EntityOperations.EG");
                data.SupportRetrieveSet = !string.IsNullOrEmpty(service.EntitiesOperationsProviderName);
                data.IsBrowseEntity = service.Name.StartsWith("Browse");
                                
                Exact.Services.MetaModel.Entity.Data.MetadataEntity x = meta.Retrieve(service.Name);
                data.CanCreate = x.OperationPermissions.Operations.Contains("Create") && !data.IsBrowseEntity;
                data.CanRetrieve = x.OperationPermissions.Operations.Contains("Retrieve") && !data.IsBrowseEntity;
                data.CanUpdate = x.OperationPermissions.Operations.Contains("Update") && !data.IsBrowseEntity;
                data.CanDelete = x.OperationPermissions.Operations.Contains("Delete") && !data.IsBrowseEntity;
                data.CanSave = data.CanCreate || data.CanUpdate;
                data.CanOpenNew = data.IsMetadataGenerated && !data.IsBrowseEntity;
                data.CanValidate = data.IsMetadataGenerated && !data.IsBrowseEntity;

                data.CanRestCreate = data.CanCreate;
                data.CanRestRetrieve = data.SupportRetrieveSet;
                data.CanRestUpdate = data.CanUpdate;
                data.CanRestDelete = data.CanDelete;

                //data.AvailableSinceRelease = Helper.ProductRelease;

                foreach (var p in x.Interface.Properties) // add metadata property for extended information.
                    data.PropertyList.Add(new Data.Property() { Name = p.Name, NameKnownInRest = GetNameKnownInRest(data.Name, p.Name) });
            }
            catch (Exception ex)
            {
                sb.AppendLine(string.Format("{0}:{1}", s.Replace("xmd\\", ""), service.Name));
                if (ex.InnerException != null) sb.AppendLine(string.Format(" - {0}", ex.InnerException.Message));
            }
        }
       
        // Add new entity property to the documentation.
        private void AddNewEntityProperty(Data.SolutionArea solution, Data.Entity data, Exact.Services.Utilities.Locator.Configuration.Service service, Exact.Services.MetaModel.Entity.Metadata meta, StringBuilder sb, string s)
        {
            try
            {
                Exact.Services.MetaModel.Entity.Data.MetadataEntity x = meta.Retrieve(service.Name);                            // Retrieve metadata
                foreach (var p in x.Interface.Properties.Where(m => !(data.PropertyList.Select(p => p.Name).Contains(m.Name)))) // Search for new properties
                {   // fill up new property.
                    data.PropertyList.Add(new Data.Property() { Name = p.Name, NameKnownInRest = GetNameKnownInRest(data.Name, p.Name)/*, AvailableSinceRelease = Helper.ProductRelease */});
                }
                PatchSavedEntityConfiguration(solution,data); // Patching new and/or missing data
            }
            catch (Exception ex)
            {
                sb.AppendLine(string.Format("{0}:{1}", s.Replace("xmd\\", ""), service.Name));
                if (ex.InnerException != null) sb.AppendLine(string.Format(" - {0}", ex.InnerException.Message));
            }
        }
        #endregion

        #region action
        // Add new action to documentation. Prefill with defaults based on available information
        private void AddNewAction(Data.Action data, Exact.Services.Utilities.Locator.Configuration.Action service, Data.SolutionArea solutionArea, Exact.Services.MetaModel.Action.Metadata meta, StringBuilder sb, string s)
        {
            // Create a new action with its properties for specific solution area.
            data = new Data.Action { Name = service.Name, ActionProviderName = service.OperationsProviderName };
            data.InputPropertyList = new Data.PropertyList();
            data.OutputPropertyList = new Data.PropertyList();
            solutionArea.ActionList.Add(data);

            try
            {   // Prefill some of the data if applicable.
                
                // Input properties
                Exact.Services.MetaModel.Action.Data.MetadataAction x = meta.RetrieveInputMetadata(service.Name);
                foreach (var p in x.Properties)
                {   // add metadata property for extended information.
                    data.InputPropertyList.Add(new Data.Property() { Name = p.Name, NameKnownInRest = GetNameKnownInRest(data.Name, p.Name) });
                }

                // Output properties
                x = meta.RetrieveOutputMetadata(service.Name);
                foreach (var p in x.Properties)
                {   // add metadata property for extended information.
                    data.OutputPropertyList.Add(new Data.Property() { Name = p.Name });
                }
            }
            catch (Exception ex)
            {
                sb.AppendLine(string.Format("{0}:{1}", s.Replace("xmd\\", ""), service.Name));
                if (ex.InnerException != null) sb.AppendLine(string.Format(" - {0}", ex.InnerException.Message));
            }
        }

        // Add new action property to the doucmentaion.
        private void AddNewActionProperty(Data.Action data, Exact.Services.Utilities.Locator.Configuration.Action service, Exact.Services.MetaModel.Action.Metadata meta, StringBuilder sb, string s)
        {
            try
            {
                Exact.Services.MetaModel.Action.Data.MetadataAction x = meta.RetrieveInputMetadata(service.Name);
                if (data.InputPropertyList == null) data.InputPropertyList = new Data.PropertyList();
                foreach (var p in x.Properties.Where(m => !(data.InputPropertyList.Select(p => p.Name).Contains(m.Name))))  // Scan for new input properties
                {   // fill up new property.
                    data.InputPropertyList.Add(new Data.Property() { Name = p.Name, NameKnownInRest = GetNameKnownInRest(data.Name, p.Name) });
                }
                
                x = meta.RetrieveOutputMetadata(service.Name);
                if (data.OutputPropertyList == null) data.OutputPropertyList = new Data.PropertyList();
                foreach (var p in x.Properties.Where(m => !(data.OutputPropertyList.Select(p => p.Name).Contains(m.Name)))) // Scan for new output properties
                {   // fill up new property.
                    data.OutputPropertyList.Add(new Data.Property() { Name = p.Name, NameKnownInRest = GetNameKnownInRest(data.Name, p.Name) });
                }
            }
            catch (Exception ex)
            {
                sb.AppendLine(string.Format("{0}:{1} ({2})", s.Replace("xmd\\", ""), service.Name, ex.Message));
                if (ex.InnerException != null) sb.AppendLine(string.Format(" - {0}", ex.InnerException.Message));
            }
        }
        #endregion
        // Compute the name known in REST, whereby, "." is not allowed and property name cannot be the same with the entity name as it is treated as a constructor in C#.
        private string GetNameKnownInRest(string entityName, string propertyName)
        {
            string nameKnownInRest = propertyName;
            if (entityName == propertyName) nameKnownInRest = String.Format("{0}Property", propertyName); // When property name is the same as entity name, Property is appended to the property name.
            return nameKnownInRest.Replace(".", "_").Replace(" ", "_"); // Replace "." with "_".
        }
        
        // Patching missing/new data to documentation.
        private void PatchSavedEntityConfiguration(Data.SolutionArea solution, Data.Entity data)
        {
            if (solution.CurrentVersion(Data.FeatureEnum.ImportFromServiceLocator) < 1)
            {
                // Version 1, added documentation for REST. Prefilling defaults for REST properties for existing entity in documentation.
                data.CanRestCreate = data.CanCreate;
                data.CanRestRetrieve = data.SupportRetrieveSet;
                data.CanRestUpdate = data.CanUpdate;
                data.CanRestDelete = data.CanDelete;

                // Version 1, added REST name into documentation. 
                foreach (var p in data.PropertyList)
                {   // patch rest name for existing properties in documentation.
                    p.NameKnownInRest = GetNameKnownInRest(data.Name, p.Name);
                }                
            }
        }

        // Start importing process...
        private void ImportClick(object sender, RoutedEventArgs e)
        {
            StringBuilder sb = new StringBuilder();
            Exact.Services.MetaModel.Entity.Metadata meta = new Exact.Services.MetaModel.Entity.Metadata();
            Exact.Services.MetaModel.Action.Metadata metaAction = new Exact.Services.MetaModel.Action.Metadata();
            
            solutionProgress.Start(0,lst.SelectedItems.Count,"");
            // for each selection configurations
            foreach (string s in lst.SelectedItems)
            {
                Data.SolutionArea solutionArea = null;
                solutionArea = Helper.Header.SolutionArea.FirstOrDefault(sa => sa.ServiceLocator == s);

                solutionProgress.Progress(string.Format("Solution: {0}", s));

                if (solutionArea == null)
                {   // instantiate new solution area if not found.
                    solutionArea = new Data.SolutionArea(true) { ServiceLocator = s, Name = s };
                    solutionArea.EntityList = new Data.EntityList();
                    solutionArea.ActionList = new Data.ActionList();
                    Helper.Header.SolutionArea.Add(solutionArea);
                }

                entityProgress.Start(0, Exact.Services.Utilities.Locator.ServiceLocator.Current(s).Services.Count, "");
                // get entity of current solution area.
                foreach (var service in Exact.Services.Utilities.Locator.ServiceLocator.Current(s).Services)
                {    
                    entityProgress.Progress(string.Format("Entity: {0} ", service.Name));
                    Data.Entity data = null;
                    data = solutionArea.EntityList.FirstOrDefault(l => l.Name == service.Name);

                    if (data == null)
                        AddNewEntity(data, service, solutionArea, meta, sb, s); // instantiate new Entity if not found.
                    else 
                        AddNewEntityProperty(solutionArea, data, service, meta, sb, s);

                    if (!string.IsNullOrEmpty(service.OperationsProviderName))
                    {   // add new entity provider
                        Data.EntityProvider entityProvider = null;
                        if (Helper.Header.EntityProvider == null) Helper.Header.EntityProvider = new Data.EntityProviderList();
                        entityProvider = Helper.Header.EntityProvider.FirstOrDefault( ep => ep.Name == service.OperationsProviderName);
                        if (entityProvider == null) Helper.Header.EntityProvider.Add(new Data.EntityProvider() { Name = service.OperationsProviderName });
                    }
                    
                    if (!string.IsNullOrEmpty(service.EntitiesOperationsProviderName))
                    {   // add new entities provider
                        Data.EntitiesProvider entitiesProvider = null;
                        if (Helper.Header.EntitiesProvider == null) Helper.Header.EntitiesProvider = new Data.EntitiesProviderList();
                        entitiesProvider = Helper.Header.EntitiesProvider.FirstOrDefault(ep => ep.Name == service.EntitiesOperationsProviderName);
                        if (entitiesProvider == null) Helper.Header.EntitiesProvider.Add(new Data.EntitiesProvider() { Name = service.EntitiesOperationsProviderName });
                    }
                }

                if (solutionArea.ActionList == null) solutionArea.ActionList = new Data.ActionList();
                // get actions of current solution area.
                foreach (Exact.Services.Utilities.Locator.Configuration.Action service in Exact.Services.Utilities.Locator.ServiceLocator.Current(s).Actions)
                {
                    Data.Action data = null;
                    data = solutionArea.ActionList.FirstOrDefault(l => l.Name == service.Name);
                    if (data == null)
                        AddNewAction(data, service, solutionArea, metaAction, sb, s); // instantiate new Entity if not found.
                    else
                        AddNewActionProperty(data, service, metaAction, sb, s);

                    if (!string.IsNullOrEmpty(service.OperationsProviderName))
                    {   // add action provider
                        Data.ActionProvider actionProvider = null;
                        if (Helper.Header.ActionProvider == null) Helper.Header.ActionProvider = new Data.ActionProviderList();
                        actionProvider = Helper.Header.ActionProvider.FirstOrDefault(ep => ep.Name == service.OperationsProviderName);
                        if (actionProvider == null) Helper.Header.ActionProvider.Add(new Data.ActionProvider() { Name = service.OperationsProviderName });
                    }
                }
                solutionArea.CurrentVersionVerified(Data.FeatureEnum.ImportFromServiceLocator); // Update versioning once import for specific solution is completed.
            }

            if (sb.Length > 0) MessageBox.Show(sb.ToString(), string.Format("{0} -Error found", this.Title), MessageBoxButton.OK, MessageBoxImage.Warning);            
            MessageBox.Show("Done!",this.Title,MessageBoxButton.OK,MessageBoxImage.Information);
            solutionProgress.Stop();
            entityProgress.Stop();
        }
    }
}
